package com.muyu.remote.feign;

import com.muyu.common.core.domain.Result;
import com.muyu.common.domain.DataName;
import com.muyu.common.domain.TableFie;
import com.muyu.remote.feign.factory.BigDataFeignFactory;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;

/**
 * @Author：张腾
 * @Package：com.muyu.remote.feign
 * @Project：cloud-integration-
 * @name：BigDataFeignService
 * @Date：2024/8/29 20:10
 */
@FeignClient(name = "cloud-property",fallbackFactory = BigDataFeignFactory.class)
public interface BigDataFeignService {
    /**
     *  连接数据库获取当前所有的数据库名称
     * @return 所有数据库以及所有表的结果集
     */
    @PostMapping("/dataRunName/extractDataName")
    @Operation(summary = "获取数据库名",description = "返回所有数据库以及所有表的结果集")
    public Result<List<DataName>> getDatabase();

    /**
     * 功能:根据表名查询所有字段信息
     * @param tableName 表名
     * @return 字段集合
     */
    @PostMapping("/tableField/selectByTableName")
    @Operation(summary = "根据表名获取所有字段")
    public Result<List<TableFie>> selectByName(@RequestParam("tableName") String tableName);
}
