package com.muyu.servier.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.enums.SysRuleActivate;
import com.muyu.common.core.exception.ServiceException;
import com.muyu.common.domain.*;
import com.muyu.common.domain.req.RuleVersionUpdReq;
import com.muyu.common.domain.resp.RuleVersionResp;
import com.muyu.compile.FilesCompilerLoad;
import com.muyu.constant.GenerateConstant;
import com.muyu.constant.MethodSuffix;
import com.muyu.engine.basic.BasicEngine;
import com.muyu.load.OSSFileLoad;
import com.muyu.mapper.RuleVersionMapper;
import com.muyu.remote.feign.BigDataFeignService;
import com.muyu.remote.feign.SelectFeignService;
import com.muyu.servier.RuleDataService;
import com.muyu.servier.RuleVersionService;
import com.muyu.upload.ALiYunUpload;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @Author：蓬叁
 * @Package：com.muyu.servier.impl
 * @Project：cloud-rule
 * @name：RuleVersionServiceImpl
 * @Date：2024/8/27 上午11:47
 */
@Log4j2
@Service
public class RuleVersionServiceImpl
        extends ServiceImpl<RuleVersionMapper, RuleVersion>
        implements RuleVersionService {

    @Autowired private RuleVersionMapper ruleVersionMapper;
    @Autowired private RuleDataService ruleDataService;
    @Autowired private SelectFeignService selectFeignService;
    @Autowired private BigDataFeignService bigDataFeignService;

    public static final String ClassPath = "/home/lib/com/muyu/generate/";

    static Map<String , BasicEngine<DataValue>> engineValueMap = new ConcurrentHashMap<>();
    static Map<String , BasicEngine<DataValue[]>> engineRecordMap = new ConcurrentHashMap<>();
    static Map<String , BasicEngine<DataValue[][]>> engineSetMap = new ConcurrentHashMap<>();

    @Override
    public List<RuleVersionResp> selectList(Long ruleId) {
        List<RuleVersionResp> ruleVersion = ruleVersionMapper.selectByIdList(ruleId);
        return ruleVersion;
    }

    @Override
    public void updateStatus(Long ruleVersionId, RuleVersionUpdReq ruleVersionUpdReq) {
        ruleVersionMapper.updateStatus(ruleVersionId,ruleVersionUpdReq);
    }

    @Override
    public void unstart(Long ruleVersionId) {
        this.settingVersionActivate(ruleVersionId, SysRuleActivate.UNSTART.getCode());
    }

    @Override
    public void start(Long ruleVersionId) {
        this.settingVersionActivate(ruleVersionId, SysRuleActivate.START.getCode());
    }

    @Override
    public void settingVersionActivate(Long ruleVersionId, String activate) {
        LambdaQueryWrapper<RuleVersion> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(RuleVersion::getRuleVersionId,ruleVersionId);
        boolean isExists = this.exists(queryWrapper);
        if (!isExists) {
            throw new ServiceException("操作规则不存在");
        }
        if (!SysRuleActivate.isCode(activate)){
            throw new ServiceException("设置状态值违法");
        }
        LambdaUpdateWrapper<RuleVersion> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.eq(RuleVersion::getRuleVersionId,ruleVersionId);
        updateWrapper.set(RuleVersion::getRuleVersionActivate,activate);
        this.update(updateWrapper);
    }

    @Override
    public RuleVersion generatedCode(RuleVersion ruleVersion) {
        RuleData ruleData = ruleDataService.getById(ruleVersion.getRuleId());
        ruleVersion.setRuleVersionType(getClassName("generate"+"_"+ruleData.getRuleCoding()+"_"+ruleVersion.getRuleVersionCode()));
        ruleVersion.setRuleVersionText(GenerateConstant.generateConstant(ruleData,ruleVersion));
        return ruleVersion;
    }

    @Override
    public void valueTest(ValueData valueData) {

        try {
            OSSFileLoad.streamingJavaDownload(valueData.getRuleVersionType());
        } catch (IOException e) {
            log.info(e.getMessage());
        }

        //编译class
        System.out.println("执行编译");
        FilesCompilerLoad.javaCompiler(valueData.getRuleVersionType());

        //上传class
        log.info("上传OSS:class文件");
        ALiYunUpload.uploadClassFiles(
                valueData.getRuleVersionType(),
                ClassPath+valueData.getRuleVersionType()+ MethodSuffix.CLASSSUFFIX);
        log.info("开始拉取....");

        //下载class
        try {
            OSSFileLoad.streamingClassDownload(valueData.getRuleVersionType());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        //对路径里的.class文件进行编译
        log.info("执行加载class:[{}]",valueData.getRuleVersionType());
        Result<Class<?>> classResult = FilesCompilerLoad.classLoad(valueData.getRuleVersionType());
        Class<?> clazz = classResult.getData();

        try {

            Object instance = clazz.getDeclaredConstructor().newInstance();
            engineValueMap.put(valueData.getRuleVersionType(), (BasicEngine<DataValue>) instance);

        } catch (InstantiationException | IllegalAccessException | InvocationTargetException | NoSuchMethodException e) {
            throw new RuntimeException(e);
        }

        detection(valueData.getRuleVersionType(),valueData.getDataValue());

    }

    @Override
    public List<DataName> selectDatabase() {
        Result<List<DataName>> database = bigDataFeignService.getDatabase();
        log.info("查询数据库数据:[{}]",database.getData());
        return database.getData();
    }

    @Override
    public List<TableNames> selectTableShow(String dataTableName) {
        Result<List<TableNames>> selectTableShow = selectFeignService.findSelectTableShow(dataTableName);
        log.info("表名称:[{}]",selectTableShow);
        log.info("表名称:[{}]",selectTableShow.getData());
        return selectTableShow.getData();
    }

    @Override
    public void uploadOSSJava(String ruleVersionType, String ruleVersionText) {
        //上传java文件
        log.info("上传OSS:java文件");
        Result<Object> objectResult = ALiYunUpload.uploadJavaFiles(ruleVersionText, ruleVersionType);
        log.info(objectResult.getData());
    }

    @Override
    public RuleVersion selectAll(String RuleVersionType) {
        return ruleVersionMapper.selectAll(RuleVersionType);
    }

    @Override
    public List<RuleVersion> selectValue() {
        return ruleVersionMapper.selectValue();
    }

    @Override
    public List<RuleVersion> selectRecord() {
        return ruleVersionMapper.selectRecord();
    }

    @Override
    public List<RuleVersion> selectSet() {
        return ruleVersionMapper.selectSet();
    }

    @Override
    public void rowTest(DataRecord dataRecord) {

        try {
            OSSFileLoad.streamingJavaDownload(dataRecord.getRuleVersionType());
        } catch (IOException e) {
            log.info(e.getMessage());
        }

        //编译class
        System.out.println("执行编译");
        FilesCompilerLoad.javaCompiler(dataRecord.getRuleVersionType());

        //上传class
        log.info("上传OSS:class文件");
        ALiYunUpload.uploadClassFiles(
                dataRecord.getRuleVersionType(),
                ClassPath+dataRecord.getRuleVersionType()+ MethodSuffix.CLASSSUFFIX);
        log.info("开始拉取....");

        //下载class
        try {
            OSSFileLoad.streamingClassDownload(dataRecord.getRuleVersionType());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        //对路径里的.class文件进行编译
        log.info("执行加载class:[{}]",dataRecord.getRuleVersionType());
        Result<Class<?>> classResult = FilesCompilerLoad.classLoad(dataRecord.getRuleVersionType());
        Class<?> clazz = classResult.getData();

        try {

            Object instance = clazz.getDeclaredConstructor().newInstance();
            engineRecordMap.put(dataRecord.getRuleVersionType(), (BasicEngine<DataValue[]>) instance);

        } catch (InstantiationException | IllegalAccessException | InvocationTargetException | NoSuchMethodException e) {
            throw new RuntimeException(e);
        }

        System.out.println("测试数据"+dataRecord);
        log.info("测试数据:[{}]",dataRecord.getDataValues());

        detection(dataRecord.getRuleVersionType(),dataRecord.getDataValues());

    }



    @Override
    public void setTest(DataSet dataSet) {

        try {
            OSSFileLoad.streamingJavaDownload(dataSet.getRuleVersionType());
        } catch (IOException e) {
            log.info(e.getMessage());
        }

        //编译class
        System.out.println("执行编译");
        FilesCompilerLoad.javaCompiler(dataSet.getRuleVersionType());

        //上传class
        log.info("上传OSS:class文件");
        ALiYunUpload.uploadClassFiles(
                dataSet.getRuleVersionType(),
                ClassPath+dataSet.getRuleVersionType()+ MethodSuffix.CLASSSUFFIX);
        log.info("开始拉取....");

        //下载class
        try {
            OSSFileLoad.streamingClassDownload(dataSet.getRuleVersionType());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        //对路径里的.class文件进行编译
        log.info("执行加载class:[{}]",dataSet.getRuleVersionType());
        Result<Class<?>> classResult = FilesCompilerLoad.classLoad(dataSet.getRuleVersionType());
        Class<?> clazz = classResult.getData();

        try {

            Object instance = clazz.getDeclaredConstructor().newInstance();
            engineSetMap.put(dataSet.getRuleVersionType(), (BasicEngine<DataValue[][]>) instance);

        } catch (InstantiationException | IllegalAccessException | InvocationTargetException | NoSuchMethodException e) {
            throw new RuntimeException(e);
        }

        detection(dataSet.getRuleVersionType(),dataSet.getDataValue());

    }

    public static void detection(String methodName,DataValue dataValue){
        BasicEngine<DataValue> engine = engineValueMap.get(methodName);
        engine.set(dataValue);
        engine.execution();
    }

    public static void detection(String methodName,DataValue[] dataValue){
        BasicEngine<DataValue[]> engine = engineRecordMap.get(methodName);
        engine.set(dataValue);
        engine.execution();
    }

    public static void detection(String methodName,DataValue[][] dataValue){
        BasicEngine<DataValue[][]> engine = engineSetMap.get(methodName);
        engine.set(dataValue);
        engine.execution();
    }

    public static String getClassName(String versionCode){
        String[] splits = versionCode.split("_");
        String className="";
        for (String split : splits) {
            className += split.substring(0,1).toUpperCase()+split.substring(1);
        }
        return className;
    }

}
